"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.doFeaturesTestCommand = void 0;
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const os_1 = require("os");
const log_1 = require("../../spec-utils/log");
const devContainers_1 = require("../devContainers");
const devContainersSpecCLI_1 = require("../devContainersSpecCLI");
const utils_1 = require("./utils");
function fail(msg) {
    log(msg, { prefix: '[-]', stderr: true });
    process.exit(1);
}
function log(msg, options) {
    const prefix = (options === null || options === void 0 ? void 0 : options.prefix) || '[+]';
    const output = `${prefix} ${msg}\n`;
    if (options === null || options === void 0 ? void 0 : options.stderr) {
        process.stderr.write(chalk_1.default.red(output));
    }
    else if (options === null || options === void 0 ? void 0 : options.info) {
        process.stdout.write(chalk_1.default.bold.blue(output));
    }
    else {
        process.stdout.write(chalk_1.default.green(output));
    }
}
function printFailedTest(feature) {
    log(`TEST FAILED:  ${feature}`, { prefix: '[-]', stderr: true });
}
async function doFeaturesTestCommand(cliHost, baseImage, pathToCollection, commaSeparatedFeatures, quiet) {
    process.stdout.write(`
┌ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┐
|    dev container 'features' |   
│     Testing Tool v0.0.0     │
└ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘\n\n`);
    log(`baseImage:         ${baseImage}`);
    log(`pathToCollection:  ${pathToCollection}`);
    log(`features:          ${commaSeparatedFeatures}`);
    const features = commaSeparatedFeatures.split(',');
    if (features.length === 0) {
        fail('No features specified\n');
    }
    // 1. Generate temporary project with 'baseImage' and all the 'features..'
    const workspaceFolder = await generateProject(cliHost, baseImage, pathToCollection, features);
    log(`workspaceFolder:   ${workspaceFolder}`);
    // 1.5. Provide a way to pass options nicely via CLI (or have test config file maybe?)
    // 2. Use  'devcontainer-cli up'  to build and start a container
    log('\n>>> Building test container... <<<\n\n', { prefix: ' ', info: true });
    await launchProject(workspaceFolder, quiet);
    // 3. Exec test script for each feature, in the provided order.
    const testResults = [];
    for (const feature of features) {
        const testScriptPath = path_1.default.join(pathToCollection, feature, 'test.sh');
        if (!(await cliHost.isFile(testScriptPath))) {
            fail(`Feature ${feature} does not have a test script!`);
        }
        // Move the test script into the workspaceFolder
        const testScript = await cliHost.readFile(testScriptPath);
        const remoteTestScriptName = `${feature}-test-${Date.now()}.sh`;
        await cliHost.writeFile(`${workspaceFolder}/${remoteTestScriptName}`, testScript);
        // Execute Test
        log('>>> Executing test(s)... <<<\n\n', { prefix: ' ', info: true });
        const result = await execTest(remoteTestScriptName, workspaceFolder);
        testResults.push({
            feature,
            result,
        });
    }
    // 4. Check for
    const allPassed = testResults.every((x) => x.result);
    if (!allPassed) {
        testResults.filter((x) => !x.result).forEach((x) => {
            printFailedTest(x.feature);
        });
    }
    else {
        log('All tests passed!');
    }
    process.exit(allPassed ? 0 : 1);
}
exports.doFeaturesTestCommand = doFeaturesTestCommand;
const devcontainerTemplate = `
{
    "image": "#{IMAGE}",
    "features": {
        #{FEATURES}
    }
}`;
async function createTempDevcontainerFolder(cliHost) {
    const systemTmpDir = (0, os_1.tmpdir)();
    const tmpFolder = path_1.default.join(systemTmpDir, 'vsch', 'container-features-test', Date.now().toString());
    await cliHost.mkdirp(`${tmpFolder}/.devcontainer`);
    return tmpFolder;
}
async function generateProject(cliHost, baseImage, basePathToCollection, featuresToTest) {
    const tmpFolder = await createTempDevcontainerFolder(cliHost);
    const features = featuresToTest
        .map((x) => `"${basePathToCollection}/${x}": "latest"`)
        .join(',\n');
    let template = devcontainerTemplate
        .replace('#{IMAGE}', baseImage)
        .replace('#{FEATURES}', features);
    await cliHost.writeFile(`${tmpFolder}/.devcontainer/devcontainer.json`, Buffer.from(template));
    return tmpFolder;
}
async function launchProject(workspaceFolder, quiet) {
    const options = {
        ...utils_1.staticProvisionParams,
        workspaceFolder,
        logLevel: log_1.LogLevel.Info,
        mountWorkspaceGitRoot: true,
        idLabels: [
            `devcontainer.local_folder=${workspaceFolder}`
        ],
        remoteEnv: {},
        log: ((_msg) => quiet ? null : process.stdout.write(_msg))
    };
    const disposables = [];
    let containerId = '';
    let remoteUser = '';
    if (quiet) {
        let isResolved = false;
        const p = (0, devContainers_1.launch)(options, disposables);
        p.then(function (res) {
            isResolved = true;
            containerId = res.containerId;
            remoteUser = res.remoteUser;
        });
        while (!isResolved) {
            // Just so visual progress with dots
            process.stdout.write('.');
            await new Promise((resolve) => setTimeout(resolve, 500));
        }
        process.stdout.write('\n');
    }
    else {
        // Stream all the container setup logs.
        const launchResult = await (0, devContainers_1.launch)(options, disposables);
        containerId = launchResult.containerId;
        remoteUser = launchResult.remoteUser;
    }
    log(`Launched container '${containerId}' as remote user ${remoteUser} \n`);
}
async function execTest(remoteTestScriptName, workspaceFolder) {
    let cmd = 'chmod';
    let args = ['777', `./${remoteTestScriptName}`];
    await exec(cmd, args, workspaceFolder);
    cmd = `./${remoteTestScriptName}`;
    args = [];
    return await exec(cmd, args, workspaceFolder);
}
async function exec(cmd, args, workspaceFolder) {
    const execArgs = {
        ...utils_1.staticExecParams,
        'workspace-folder': workspaceFolder,
        cmd,
        args,
        _: [
            cmd,
            ...args
        ]
    };
    const result = await (0, devContainersSpecCLI_1.doExec)(execArgs);
    return (result.outcome === 'success');
}
//# sourceMappingURL=testContainerFeatures.js.map